import axios from 'axios'
import { getToken } from './auth'
import { Message, MessageBox } from 'element-ui'
import { showLoading, closeLoading } from '@/utils/showOrHideLoading'

axios.defaults.headers.common['token'] = getToken()

/**
 * @author renwenji
 * @date 2020/1/9
 * @Description: 导出excel的方法
 * @params1: url 请求的地址
 * @params2: data 传递的数据参数
 * @returns: promise 成功resolve 失败reject
 */
export function exportExcel(url,data={}) {
  return new Promise((resolve, reject) => {
    showLoading()
    axios({
      url,
      method: 'GET',
      params:data,
      responseType: 'blob' // 非常重要
    }).then(response => {
      closeLoading()
      console.log('response', response)
      const { data } = response
      const reader = new FileReader()
      reader.readAsText(data)
      resolve(response)
    })
      .catch(error => {
        closeLoading()
        Message({
          message: '数据为空或其他错误，无法导出！',
          type: 'error',
          duration: 5 * 1000
        })
        reject(error)
      })
  })
}

// 创建对象url
function createObjectURL(blob) {
  if (window.URL) {
    return window.URL.createObjectURL(blob)
  } else if (window.webkitURL) {
    return window.webkitURL.createObjectURL(blob)
  } else {
    return null
  }
}

// 释放对象url资源
function revokeObjectURL(url) {
  if (window.URL) {
    window.URL.revokeObjectURL(url)
  } else if (window.webkitURL) {
    window.webkitURL.revokeObjectURL(url)
  }
}

/**
 * @author renwenji
 * @date 2020/1/9
 * @Description: 下载excel,动态添加a标签、下载后移除
 * @params1: response
 * @params2: fileName 下载的文件名
 */
export function downLoadExcel(response, fileName) {
  const url = createObjectURL(new Blob([response.data]))
  const link = document.createElement('a')
  link.href = url
  link.setAttribute('download', fileName)
  document.body.appendChild(link)
  link.click()
  document.body.removeChild(link)
  revokeObjectURL(url)
}

// 确认导出
export function confirmExportExcel(url, fileName) {
  MessageBox.confirm('确定进行导出操作？', '确认信息', {
    distinguishCancelAndClose: true,
    confirmButtonText: '确定',
    cancelButtonText: '取消'
  })
    .then(() => {
      exportExcel(url).then(res => {
        downLoadExcel(res, fileName)
        Message({
          message: '导出成功!',
          type: 'success',
          duration: 5 * 1000
        })
      })
    }).catch(action => {
      Message({
        message: '已取消操作!',
        type: 'info',
        duration: 5 * 1000
      })
    })
}
/**
 * formData的导出Excel
 * @param url
 * @param data
 * @returns {Promise<unknown>}
 */
 export function formDataExportExcel(url, data) {
  return new Promise((resolve, reject) => {
    axios({
      url: url,
      method: 'POST', //请求方式
      headers: { 'content-type': 'application/x-www-form-urlencoded' },
      data,
      responseType: 'blob' // 非常重要
    }).then(response => {
      const { data } = response

      // 解析错误返回的信息
      if (data.type.includes('application/json')) {
        const reader = new FileReader()
        reader.onload = function(event) {
          const content = reader.result
          const message = JSON.parse(content).message
          Message({
            message,
            type: 'error',
            duration: 5 * 1000
          })
        }
        reader.readAsText(data)
        return true
      }

      // excel格式
      // const types = ['application/vnd.ms-excel', 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet']
      // if (types.includes(data.type)) {
      resolve(response)
      // }
    }).catch(error => {
      console.log('error ==', error)
      Message({
        message: error.message,
        type: 'error',
        duration: 5 * 1000
      })
      reject(error)
    })
  })
}
